@@ -10,7 +10,7 @@ from rook.processor.namespaces.trace_namespace import TraceNamespace
 from rook.processor.namespaces.process_state_namespace import ProcessStateNamespace
 from rook.user_warnings import UserWarnings
 
-from rook.exceptions import RookRuleMaxExecutionTimeReached, RookRuleRateLimited
+from rook.exceptions import RookRuleMaxExecutionTimeReached, RookRuleRateLimited, RookGlobalRuleRateLimited
 from rook.augs.aug_rate_limiter import AugRateLimiter
 from rook.config import RateLimiter
 import rook.utils as utils
@@ -26,7 +26,8 @@ class Aug(object):
     - action - specifies the modification to preform.
     """
 
-    def __init__(self, aug_id, extractor, condition, action, max_aug_execution_time_ns=0, rate_limits_ns=None):
+    def __init__(self, aug_id, extractor, condition, action, global_rate_limit=AugRateLimiter(None, None),
+                 max_aug_execution_time_ns=0, rate_limits_ns=None):
         """Build an Aug object from the individual elements."""
         self.aug_id = aug_id
         self._extractor = extractor
@@ -34,6 +35,7 @@ class Aug(object):
         self._action = action
         self._max_aug_time = max_aug_execution_time_ns
         self._enabled = True
+        self._global_rate_limiter = global_rate_limit
 
         if rate_limits_ns:
             self._rate_limiter = AugRateLimiter(rate_limits_ns[0], rate_limits_ns[1])
@@ -50,7 +52,10 @@ class Aug(object):
             return
 
         now_ns = utils.get_most_accurate_time_stamp_nano_seconds()
-        limit_key = self._rate_limiter.allow(now_ns)
+        rate_limits_keys = self._allow(now_ns)
+
+        if Aug._enforce_rate_limits(rate_limits_keys):
+            return
 
         try:
             if self._extractor:
@@ -70,12 +75,7 @@ class Aug(object):
                 'state': ProcessStateNamespace(),
             })
 
-            # TODO - cleanup rate limit
             if not self._condition or self._condition.evaluate(namespace, extracted):
-                if limit_key is None:
-                    UserWarnings.send_warning(Error(exc=RookRuleRateLimited()))
-                    return
-
                 msg_id = uuid.uuid4().hex
 
                 logger.info("Executing aug-\t%s (msg ID %s)", self.aug_id, msg_id)
@@ -87,4 +87,23 @@ class Aug(object):
                 UserWarnings.set_error(Error(exc=RookRuleMaxExecutionTimeReached()))
                 self._enabled = False
 
-            self._rate_limiter.record(limit_key, duration)
+            self._record_rate_limits(rate_limits_keys, duration)
+
+    def _allow(self, now_ns):
+        return self._rate_limiter.allow(now_ns), self._global_rate_limiter.allow(now_ns)
+
+    @staticmethod
+    def _enforce_rate_limits(rate_limits_keys):
+        if rate_limits_keys[0] is None:
+            UserWarnings.send_warning(Error(exc=RookRuleRateLimited()))
+            return True
+
+        if rate_limits_keys[1] is None:
+            UserWarnings.send_warning(Error(exc=RookGlobalRuleRateLimited()))
+            return True
+
+        return False
+
+    def _record_rate_limits(self, rate_limits_keys, duration):
+        self._rate_limiter.record(rate_limits_keys[0], duration)
+        self._global_rate_limiter.record(rate_limits_keys[1], duration)
