@@ -1,6 +1,10 @@
 import socket
 import threading
 import time
+
+from rook.com_ws.envelope_wrappers.basic_envelope_wrapper import BasicEnvelopeWrapper
+from rook.com_ws.envelope_wrappers.basic_serialized_envelope_wrapper import BasicSerializedEnvelopeWrapper
+from rook.com_ws.envelope_wrappers.protobuf_2_envelope_wrapper import Protobuf2EnvelopeWrapper
 from six.moves.queue import Queue, Empty
 from six.moves.urllib.parse import urlparse
 import select
@@ -12,7 +16,7 @@ import os
 import six
 import websocket
 
-from rook.exceptions import RookCommunicationException, RookInvalidToken, RookDependencyConflict, RookMessageSizeExceeded, RookMissingToken
+from rook.exceptions import RookCommunicationException, RookInvalidToken, RookDependencyConflict, RookQueueSizeExceeded, RookMissingToken
 import rook.com_ws.socketpair_compat  # do not remove - adds socket.socketpair on Windows lgtm[py/unused-import]
 
 try:
@@ -79,7 +83,10 @@ class AgentCom(object):
 
         self._loop = None
         self._connection = None
+
+        # Queue hold outgoing messages wrapped with IEnvelopeWrapper
         self._queue = Queue()
+        self._queue_messages_length = 0
 
         self._running = True
 
@@ -101,16 +108,19 @@ class AgentCom(object):
         self._thread.start()
 
     def stop(self):
+        if not self._running:
+            return
+
         self._running = False
 
         if self._connection is not None:
             self._connection.close(1000)
 
+        self._thread.join()
+        self._connection = None
+
     def restart(self):
         self.stop()
-        self._thread.join()
-
-        self._connection = None
 
         self._init_connect_thread()
 
@@ -122,15 +132,24 @@ class AgentCom(object):
         self._labels = labels or {}
         self._tags = tags or []
 
+    def send_user_message(self, aug_id, message_id, arguments):
+        envelope = Protobuf2EnvelopeWrapper(self.id, aug_id, message_id, arguments)
+
+        self.add_envelope(envelope)
+
     def add(self, message):
         if self._queue.qsize() >= AgentComConfiguration.MAX_QUEUED_MESSAGES:
             return None
 
-        packaged_message = wrap_in_envelope(message)
-        if len(packaged_message) > AgentComConfiguration.MAX_MESSAGE_LENGTH:
-            return RookMessageSizeExceeded(len(packaged_message), AgentComConfiguration.MAX_MESSAGE_LENGTH)
+        envelope = BasicSerializedEnvelopeWrapper(message)
+        return self.add_envelope(envelope)
 
-        self._queue.put(packaged_message)
+    def add_envelope(self, envelope):
+        if len(envelope) + self._queue_messages_length > AgentComConfiguration.MAX_QUEUE_MESSAGES_LENGTH:
+            return RookQueueSizeExceeded(len(envelope), self._queue_messages_length, AgentComConfiguration.MAX_QUEUE_MESSAGES_LENGTH)
+
+        self._queue_messages_length += len(envelope)
+        self._queue.put(envelope)
         return None
 
     def is_queue_full(self):
@@ -235,7 +254,7 @@ class AgentCom(object):
                     pass
 
                 try:
-                    self._queue.put(outgoing_exit_sentinel)
+                    self._queue.put(BasicEnvelopeWrapper(outgoing_exit_sentinel))
                 except Exception:
                     logger.debug("Failed to signal stop", exc_info=1)
 
@@ -323,6 +342,10 @@ class AgentCom(object):
                         logger.debug("Incoming thread - socket disconnected")
                         break
 
+                    if len(msg) > AgentComConfiguration.AGENT_COM_INCOMING_MAX_MESSAGE_SIZE:
+                        logger.error("message length (%d) exceed max size", msg)
+                        continue
+
                     envelope = envelope_pb.Envelope()
                     envelope.ParseFromString(msg)
                     self._handle_incoming_message(envelope)
@@ -339,7 +362,7 @@ class AgentCom(object):
 
     def flush_all_messages(self):
         flush_event = FlushMessagesEvent()
-        self._queue.put(flush_event)
+        self._queue.put(BasicEnvelopeWrapper(flush_event))
         flush_event.event.wait(AgentComConfiguration.FLUSH_TIMEOUT)
 
     def _outgoing(self, outgoing_exit_sentinel, on_exit):
@@ -348,13 +371,16 @@ class AgentCom(object):
             self._connection.ping()
 
             while True:
-                msg = None
+                envelope_wrapper = None
                 if (time.time() - last_ping) >= AgentComConfiguration.PING_INTERVAL:
                     last_ping = time.time()
                     self._connection.ping()
 
                 try:
-                    msg = self._queue.get(timeout=AgentComConfiguration.PING_INTERVAL)
+                    envelope_wrapper = self._queue.get(timeout=AgentComConfiguration.PING_INTERVAL)
+                    if envelope_wrapper is None:
+                        break
+                    msg = envelope_wrapper.get_buffer()
                     if isinstance(msg, ExitThreadSentinel):
                         if msg is outgoing_exit_sentinel:
                             break
@@ -366,9 +392,13 @@ class AgentCom(object):
                 except Empty:
                     continue
                 except (socket.error, websocket.WebSocketConnectionClosedException):
-                    if msg is not None:
-                        self._queue.put(msg)
+                    if envelope_wrapper is not None:
+                        self._queue.put(envelope_wrapper)
                     break
+
+                if envelope_wrapper is not None:
+                    self._queue_messages_length -= len(envelope_wrapper)
+
         except:  # lgtm[py/catch-base-exception]
             try:
                 logger.exception("Outgoing thread failed")
