@@ -1,12 +1,13 @@
 import datetime
 import traceback
 import decimal
-from types import FunctionType, MethodType, TracebackType
-from collections import Iterable
+from types import TracebackType
+from six.moves.collections_abc import Iterable
 
 import six
 
 from google.protobuf.internal.type_checkers import Int32ValueChecker, Int64ValueChecker
+from rook.processor.namespace_serializer_base import NamespaceSerializerBase
 
 from .namespaces.container_namespace import ContainerNamespace
 from .namespaces.python_object_namespace import PythonObjectNamespace
@@ -22,43 +23,13 @@ from .namespaces.unknown_namespace import UnknownNamespace
 from .namespaces.code_object_namespace import CodeObjectNamespace
 from .namespaces.dynamic_object_namespace import DynamicObjectNamespace
 
-from ..logger import logger
+from rook.logger import logger
 
 from rook.protobuf import variant_pb2
 
-from rook.config import ProtobufConfiguration
-from google.protobuf.descriptor import Descriptor as google_descriptor
-
-try:
-    import numpy
-except ImportError:
-    numpy = None
-
-try:
-    import torch
-except ImportError:
-    torch = None
-
-try:
-    import multidict
-except ImportError:
-    multidict = None
-
-if six.PY2:
-    from types import TypeType
-
-
-class NamespaceSerializer(object):
+class NamespaceSerializer(NamespaceSerializerBase):
     def __init__(self, use_string_cache=False):
-        self.use_string_cache = use_string_cache
-        self.string_cache = {}
-
-        if use_string_cache:
-            # Lock the 0 index since some variant will have no originalType (container for example)
-            self.string_cache[""] = 0
-
-    def get_string_cache(self):
-        return self.string_cache
+        NamespaceSerializerBase.__init__(self, use_string_cache)
 
     def dump(self, namespace, variant, log_errors=True):
         try:
@@ -165,7 +136,7 @@ class NamespaceSerializer(object):
 
     def _dump_python_object_unsafe(self, obj, variant, current_depth, config, log_errors):
         self._dump_base_object(obj, variant, current_depth, config, log_errors)
-        if self._is_primitive(obj):
+        if isinstance(obj, NamespaceSerializerBase.PRIMITIVE_TYPES):
             self._dump_primitive(obj, str(type(obj)), variant, config.max_string)
         elif isinstance(obj, LIST_TYPE):
             self._dump_list(obj, variant, current_depth, config, log_errors)
@@ -188,26 +159,6 @@ class NamespaceSerializer(object):
         else:
             self._dump_not_supported(obj, variant)
 
-        # If debugging NamespaceSerializer
-        if ProtobufConfiguration.NAMESPACE_SERIALIZER_DUMPING:
-
-            try:
-                # We have to write to a local file as logging itself is using NamespaceSerializer
-                with open(ProtobufConfiguration.NAMESPACE_SERIALIZER_DUMPING, 'a') as f:
-                    try:
-                        f.write('Attempting dump of-\t' + str(type(obj)) + '\n')
-                        dump = variant.SerializeToString()
-                        f.write('Dump successful. Size-\t' + str(len(dump)) + '\n')
-                    except Exception:
-                        f.write('Erorr while processing object\n')
-                        f.write(traceback.format_exc())
-                        f.write('\n')
-            except IOError:
-                # We have to write to console as we have nothing else
-                print('Failed to open file')
-
-    BUILTIN_ATTRIBUTES_IGNORE = ['__dict__', '__module__', '__weakref__', '__name__', '__doc__', '__bases__']
-
     def _dump_traceback(self, obj, variant, current_depth, config, log_errors):
         # python separates the "forward" stack (callees of the except clause)
         # and the "backward" stack (callers of above)
@@ -220,14 +171,6 @@ class NamespaceSerializer(object):
         variant.string_value.original_size = len(value)
         variant.string_value.value = value
 
-    def _get_object_width(self, obj):
-        object_width = 0
-        if hasattr(obj, '__dict__') and obj.__dict__:
-            object_width += len(obj.__dict__)
-        if hasattr(obj, '__slots__') and obj.__slots__:
-            object_width += len(obj.__slots__)
-        return object_width
-
     def _dump_base_object(self, obj, variant, current_depth, config, log_errors):
         original_type = str(type(obj))
         if not self.use_string_cache:
@@ -269,20 +212,6 @@ class NamespaceSerializer(object):
                         value = None
                     self._dump_python_object(value, attribute.value, object_weight, config, log_errors)
 
-    try:
-        BINARY_TYPES = (buffer, bytearray)
-        CODE_TYPES = (FunctionType, MethodType, TypeType)
-        PRIMITIVE_TYPES = (type(None), int, long, float, str, unicode, complex, decimal.Decimal) + BINARY_TYPES + CODE_TYPES + (datetime.datetime,)
-
-    except NameError:
-        BINARY_TYPES = (bytearray, bytes)
-        CODE_TYPES = (FunctionType, MethodType, type)
-        PRIMITIVE_TYPES = (type(None), int, float, str, complex, decimal.Decimal) + BINARY_TYPES + CODE_TYPES + (datetime.datetime,)
-
-    @staticmethod
-    def _is_primitive(obj):
-        return isinstance(obj, NamespaceSerializer.PRIMITIVE_TYPES)
-
     def _dump_max_depth(self, variant):
         variant.variant_type = variant.VARIANT_MAX_DEPTH
 
@@ -478,10 +407,12 @@ class NamespaceSerializer(object):
         variant.variant_type = variant.VARIANT_MAX_DEPTH
 
     def _dump_traceback_namespace(self, namespace, variant, log_errors):
+        variant.variant_type = variant_pb2.Variant.VARIANT_TRACEBACK
+
         if not self.use_string_cache:
-            namespace.dump(variant)
+            namespace.dump(variant.traceback.locations)
         else:
-            namespace.dump(variant, self._get_string_index_in_cache)
+            namespace.dump(variant.traceback.locations, self._get_string_index_in_cache)
 
     def dump_code_namespace(self, namespace, variant, log_errors):
         self._dump_dumped_object_namespace(namespace, variant, log_errors)
@@ -625,53 +556,3 @@ class NamespaceSerializer(object):
                                    variant.code_value.lineno,
                                    variant.original_type,
                                    self._load_attributes(variant))
-
-    def _get_string_index_in_cache(self, str):
-        if str in self.string_cache:
-            return self.string_cache[str]
-
-        current_size = len(self.string_cache)
-        self.string_cache[str] = current_size
-        return current_size
-
-    @staticmethod
-    def normalize_string(obj):
-        if six.PY2:
-            if isinstance(obj, str):
-                return unicode(obj, errors="replace")
-            else:
-                return unicode(obj)
-        else:
-            return str(obj)
-
-    @staticmethod
-    def is_numpy_obj(obj):
-        if not numpy:
-            return False
-
-        return isinstance(obj, numpy.generic)
-
-    @staticmethod
-    def is_torch_obj(obj):
-        if not torch:
-            return False
-
-        module = getattr(type(obj), '__module__', None)
-        if not module:
-            return False
-
-        return module.startswith('torch')
-
-    @staticmethod
-    def is_multidict_obj(obj):
-        if not multidict:
-            return False
-
-        return isinstance(obj, (multidict._multidict.MultiDict, multidict._multidict.CIMultiDict, multidict._multidict.MultiDictProxy, multidict._multidict.CIMultiDictProxy))
-
-    @staticmethod
-    def is_protobuf_obj(obj):
-        if hasattr(obj, "DESCRIPTOR"):
-            return isinstance(obj.DESCRIPTOR, google_descriptor)
-
-        return False
