@@ -71,7 +71,7 @@ class PythonObjectNamespace(Namespace):
         def tailor_limits(cls, obj):
             if isinstance(obj, six.string_types):
                 return PythonObjectNamespace.ObjectDumpConfig(1, 0, 0, cls.UNLIMITED_STRING)
-            if isinstance(obj, (list, dict, set, tuple)):
+            if isinstance(obj, (list, dict, set, tuple)) and len(obj) > cls.TOLERANT_MAX_WIDTH:
                 return PythonObjectNamespace.ObjectDumpConfig(max_width=cls.UNLIMITED_COLLECTION_WIDTH)
             else:
                 return PythonObjectNamespace.ObjectDumpConfig.tolerant_limits(obj)
