@@ -28,8 +28,11 @@ class TracebackNamespace(Namespace):
 
         return current_frame
 
-    def dump(self, variant, get_string_index_in_cache=None):
-        variant.variant_type = variant_pb2.Variant.VARIANT_TRACEBACK
+    def dump(self, traceback_frames_holder, get_string_index_in_cache=None):
+        """
+        Dump traceback frames to given 'traceback_frames_holder'.
+        It's the caller responsibility to set the variant type.
+        """
 
         current_frame = self._frame
 
@@ -46,7 +49,7 @@ class TracebackNamespace(Namespace):
                 frame.name_index_in_cache = get_string_index_in_cache(current_frame.function().obj if current_frame.function().obj is not None else "unavailable")
                 frame.module_index_in_cache = get_string_index_in_cache(current_frame.module().obj if current_frame.module().obj is not None else "unavailable")
 
-            variant.traceback.locations.append(frame)
+            traceback_frames_holder.append(frame)
 
             current_frame = current_frame.f_back()
 
